@@ -1,7 +1,7 @@
-require 'puppet/util'
+require_relative '../../puppet/util'
 require 'monitor'
-require 'puppet/parser/parser_factory'
-require 'puppet/concurrent/lock'
+require_relative '../../puppet/parser/parser_factory'
+require_relative '../../puppet/concurrent/lock'
 
 # Just define it, so this class has fewer load dependencies.
 class Puppet::Node
@@ -166,6 +166,44 @@ class Puppet::Node::Environment
   # @api private
   attr_reader :lock
 
+  # For use with versioned dirs
+  # our environment path may contain symlinks, while we want to resolve the
+  # path while reading the manifests we may want to report the resources as
+  # coming from the configured path.
+  attr_accessor :configured_path
+
+  # See :configured_path above
+  attr_accessor :resolved_path
+
+  # Ensure the path given is of the format we want in the catalog/report.
+  #
+  # Intended for use with versioned symlinked environments. If this
+  # environment is configured with "/etc/puppetlabs/code/environments/production"
+  # but the resolved path is
+  #
+  # "/opt/puppetlabs/server/puppetserver/filesync/client/puppet-code/production_abcdef1234"
+  #
+  # this changes the filepath
+  #
+  # "/opt/puppetlabs/server/puppetserver/filesync/client/puppet-code/production_abcdef1234/modules/foo/manifests/init.pp"
+  #
+  # to
+  #
+  # "/etc/puppetlabs/code/environments/production/modules/foo/manifests/init.pp"
+  def externalize_path(filepath)
+    paths_set        = configured_path && resolved_path
+    munging_possible = paths_set && configured_path != resolved_path
+    munging_desired  = munging_possible &&
+                         Puppet[:report_configured_environmentpath] &&
+                         filepath.to_s.start_with?(resolved_path)
+
+    if munging_desired
+      File.join(configured_path, filepath.delete_prefix(resolved_path))
+    else
+      filepath
+    end
+  end
+
   # Checks to make sure that this environment did not have a manifest set in
   # its original environment.conf if Puppet is configured with
   # +disable_per_environment_manifest+ set true.  If it did, the environment's
@@ -214,15 +252,22 @@ class Puppet::Node::Environment
     errors
   end
 
+  def rich_data_from_env_conf
+    unless @checked_conf_for_rich_data
+      environment_conf = Puppet.lookup(:environments).get_conf(name)
+      @rich_data_from_conf = environment_conf&.rich_data
+      @checked_conf_for_rich_data = true
+    end
+    @rich_data_from_conf
+  end
+
   # Checks if this environment permits use of rich data types in the catalog
+  # Checks the environment conf for an override on first query, then going forward
+  # either uses that, or if unset, uses the current value of the `rich_data` setting.
   # @return [Boolean] `true` if rich data is permitted.
   # @api private
   def rich_data?
-    if @rich_data.nil?
-      environment_conf = Puppet.lookup(:environments).get_conf(name)
-      @rich_data = (environment_conf.nil? ? Puppet[:rich_data] : environment_conf.rich_data)
-    end
-    @rich_data
+    @rich_data = rich_data_from_env_conf.nil? ? Puppet[:rich_data] : rich_data_from_env_conf
   end
 
   # Return an environment-specific Puppet setting.
@@ -266,7 +311,7 @@ class Puppet::Node::Environment
   # @param name [String] The module name
   # @return [Puppet::Module, nil] The module if found, else nil
   def module(name)
-    modules.find {|mod| mod.name == name}
+    modules_by_name[name]
   end
 
   # Locate a module instance by the full forge name (EG authorname/module)
@@ -298,14 +343,16 @@ class Puppet::Node::Environment
     if @modules.nil?
       module_references = []
       project = Puppet.lookup(:bolt_project) { nil }
-      seen_modules = if project
+      seen_modules = if project && project.load_as_module?
                        module_references << project.to_h
                        { project.name => true }
                      else
                        {}
                      end
       modulepath.each do |path|
-        Dir.entries(path).each do |name|
+        Puppet::FileSystem.children(path).map do |p|
+          Puppet::FileSystem.basename_string(p)
+        end.each do |name|
           next unless Puppet::Module.is_module_directory?(name, path)
           warn_about_mistaken_path(path, name)
           if not seen_modules[name]
@@ -327,6 +374,12 @@ class Puppet::Node::Environment
     @modules
   end
 
+  # @api private
+  def modules_by_name
+    @modules_by_name ||= Hash[modules.map { |mod| [mod.name, mod] }]
+  end
+  private :modules_by_name
+
   # Generate a warning if the given directory in a module path entry is named `lib`.
   #
   # @api private
@@ -335,17 +388,15 @@ class Puppet::Node::Environment
   # @param name [String] The directory name
   def warn_about_mistaken_path(path, name)
     if name == "lib"
-      Puppet.debug("Warning: Found directory named 'lib' in module path ('#{path}/lib'); unless " +
-          "you are expecting to load a module named 'lib', your module path may be set " +
-          "incorrectly.")
+      Puppet.debug {
+        "Warning: Found directory named 'lib' in module path ('#{path}/lib'); unless you \
+        are expecting to load a module named 'lib', your module path may be set incorrectly."
+      }
     end
   end
 
   # Modules broken out by directory in the modulepath
   #
-  # @note This method _changes_ the current working directory while enumerating
-  #   the modules. This seems rather dangerous.
-  #
   # @api public
   #
   # @return [Hash<String, Array<Puppet::Module>>] A hash whose keys are file
@@ -354,13 +405,13 @@ class Puppet::Node::Environment
     modules_by_path = {}
     modulepath.each do |path|
       if Puppet::FileSystem.exist?(path)
-        Dir.chdir(path) do
-          module_names = Dir.entries(path).select do |name|
-            Puppet::Module.is_module_directory?(name, path)
-          end
-          modules_by_path[path] = module_names.sort.map do |name|
-            Puppet::Module.new(name, File.join(path, name), self)
-          end
+        module_names = Puppet::FileSystem.children(path).map do |p|
+          Puppet::FileSystem.basename_string(p)
+        end.select do |name|
+          Puppet::Module.is_module_directory?(name, path)
+        end
+        modules_by_path[path] = module_names.sort.map do |name|
+          Puppet::Module.new(name, File.join(path, name), self)
         end
       else
         modules_by_path[path] = []
@@ -540,10 +591,11 @@ class Puppet::Node::Environment
       if file == NO_MANIFEST
         empty_parse_result
       elsif File.directory?(file)
+        # We sort the results because Dir.glob order is inconsistent in Ruby < 3 (see PUP-10115).
         parse_results = Puppet::FileSystem::PathPattern.absolute(File.join(file, '**/*.pp')).glob.sort.map do | file_to_parse |
           parser.file = file_to_parse
           parser.parse
-          end
+        end
         # Use a parser type specific merger to concatenate the results
         Puppet::Parser::AST::Hostclass.new('', :code => Puppet::Parser::ParserFactory.code_merger.concatenate(parse_results))
       else
